Package: qsmooth
Title: Smooth quantile normalization
Version: 1.28.0
Authors@R: c(
    person(c("Stephanie", "C."), "Hicks", role = c("aut", "cre"), 
		email = "shicks19@jhu.edu", 
		comment = c(ORCID = "0000-0002-7858-0231")),
    person("Kwame", "Okrah", role = "aut"),
    person("Koen", "Van den Berge", role = "ctb"),
    person("Hector", "Corrada Bravo", role = "aut", 
		comment = c(ORCID = "0000-0002-1255-4444")), 
    person("Rafael", "Irizarry", role = "aut", 
		comment = c(ORCID = "0000-0002-3944-4309")))
Imports: SummarizedExperiment, utils, sva, stats, methods, graphics,
        Hmisc
Depends: R (>= 4.0)
Suggests: bodymapRat, quantro, knitr, rmarkdown, BiocStyle, testthat
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Encoding: UTF-8
License: GPL-3
Description: Smooth quantile normalization
    is a generalization of quantile normalization,
    which is average of the two types of assumptions
    about the data generation process: quantile
    normalization and quantile normalization between
    groups.
biocViews: Normalization, Preprocessing, MultipleComparison,
        Microarray, Sequencing, RNASeq, BatchEffect
git_url: https://git.bioconductor.org/packages/qsmooth
git_branch: RELEASE_3_23
git_last_commit: d84931f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:02:24 UTC; biocbuild
Author: Stephanie C. Hicks [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7858-0231>),
  Kwame Okrah [aut],
  Koen Van den Berge [ctb],
  Hector Corrada Bravo [aut] (ORCID:
    <https://orcid.org/0000-0002-1255-4444>),
  Rafael Irizarry [aut] (ORCID: <https://orcid.org/0000-0002-3944-4309>)
Maintainer: Stephanie C. Hicks <shicks19@jhu.edu>

# Generated by roxygen2: do not edit by hand

export(qsmooth)
export(qsmoothGC)
export(qsmoothPlotWeights)
export(qstats)
exportClasses(qsmooth)
exportMethods(qsmoothData)
exportMethods(qsmoothWeights)
import(methods)
importFrom(Hmisc,cut2)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(graphics,abline)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(stats,ave)
importFrom(stats,model.matrix)
importFrom(stats,runmed)
importFrom(sva,ComBat)
importFrom(utils,head)
importFrom(utils,tail)

# qsmooth

### Global normalization methods and their assumptions

Global normalization methods such as quantile normalization have become a standard part of the
analysis pipeline for high-throughput data to remove unwanted technical variation. These 
methods and others that rely solely on observed data without external information (e.g. spike-ins)
are based on the assumption that only a minority of genes are expected to be differentially
expressed (or that an equivalent number of genes increase and decrease across biological conditions. 
This assumption can be interpreted in different ways leading to different global normalization 
procedures. For example, in one normalization procedure, the method assumes
the mean expression level across genes should be the same across samples. 
In contrast, quantile normalization assumes the only difference between the statistical distribution
of each sample is technical variation. Normalization is achieved by forcing the observed distributions
to be the same and the average distribution, obtained by taking the average of each quantile 
across samples, is used as the reference. 

### How to evaluate if global normalization methods are appropriate? 

While these assumptions may be reasonable in certain experiments, they may not always be 
appropriate. Recently, an R/Bioconductor package ([`quantro`](http://www.bioconductor.org/packages/release/bioc/html/quantro.html))
has been developed to test for global differences between groups of distributions to evaluate whether 
global normalization methods such as quantile normalization should be applied. If global differences
are found between groups of distributions, these changes may be of technical or biological of interest.
If these changes are of technical interest (e.g. batch effects), then global normalization methods should be applied. 
If these changes are related to a biological factor (e.g. normal/tumor or two tissues), then 
global normalization methods should not be applied because the methods will remove the interesting biological variation 
(i.e. differentially expressed genes) and artificially induce differences between genes that were not 
differentially expressed. In the cases with global differences between groups of distributions 
between biological conditions, quantile normalization is not an appropriate normalization method. In 
these cases, we can consider **a more relaxed assumption** about the data, namely that the statistical distribution
of each sample should be the same within biological conditions or groups (compared to the more 
stringent assumption of quantile normalization, which states the statistical distribution is the same across all samples).

### qsmooth: a generalization of quantile normalization

Here we introduce a generalization of quantile normalization, referred to as `smooth quantile normalization`
(**qsmooth**), which is a weighted average of the two types of assumptions about the data. 
The **qsmooth** R-package contains the `qsmooth()` function, which computes a weight at every quantile
that compares the variability between groups relative to within groups. In one extreme, quantile normalization 
is applied and in the other extreme quantile normalization within each biological condition is applied.
The weight shrinks the group-level quantile normalized data towards the overall reference quantiles 
if variability between groups is sufficiently smaller than the variability within groups. The algorithm is described in the
Figure below (see the [`vignettes/qsmooth-vignette.pdf`](https://github.com/stephaniehicks/qsmooth/blob/master/vignettes/qsmooth-vignette.pdf) for more details). 

![qsmooth algorithm](https://github.com/stephaniehicks/qsmooth/raw/master/vignettes/qsmooth_algo.jpg)

### Installing qsmooth

The R-package **qsmooth** can be installed from Github using the R 
package [devtools](https://github.com/hadley/devtools): 


Use  to install the latest version of **qsmooth** from Github:
```s
library(devtools)
install_github("stephaniehicks/qsmooth")
```

It can also be installed using Bioconductor: 

```s
# install BiocManager from CRAN (if not already installed)
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")

# install qsmooth package
BiocManager::install("qsmooth")
```

After installation, the package can be loaded into R.
```s
library(qsmooth)
```


# Using qsmooth

The main function in the **qsmooth** package is `qsmooth()`.  The `qsmooth()` function needs two objects: 
(1) a data frame or matrix with observations (e.g. probes or genes) on the rows and samples as the columns 
(e.g. let's call it `eset`) and (2) a group level factor called `group_factor` (let's call it `outcome`). 
This order of this factor variable must match the order of the columns in the `eset` object because it contains
 information about which group each sample is from.

To run the `qsmooth()` function, 
```
qs <- qsmooth(object = eset, group_factor = outcome)
```
Individual slots can be extracted using accessor methods:
```
qsmoothData(qs) # extract smoothed quantile normalized data
qsmoothWeights(qs) # extract smoothed quantile normalized weights
```

The weights can be directly plotted using the `qsmoothPlotWeights()` function. 
```
qsmoothPlotWeights(qs) # plot weights 
```

See `vignettes/qsmooth-vignette.pdf` for more details. 

# Bug reports
Report bugs as issues on the [GitHub repository](https://github.com/stephaniehicks/qsmooth)


# Contributors

* [Stephanie Hicks](https://github.com/stephaniehicks)
* [Kwame Okrah](https://github.com/kokrah)

#' @title qstats 
#'
#' @description This function is a helper function that 
#' computes quantile statistics for the function 
#' \code{qsmooth}. 
#'
#' @param object an object which is a data frame or 
#' matrix with observations (e.g. probes or genes) on 
#' the rows and samples as the columns.  
#' @param group_factor a group level continuous or categorial 
#' covariate associated with each sample or column in the 
#' \code{object}. The order of the \code{group_factor} must 
#' match the order of the columns in \code{object}. 
#' @param window window size for running median which is 
#' a fraction of the number of rows in \code{object}. 
#' Default is 0.05. 
#' 
#' @return A list of quantile statistics including 
#' \item{Q}{sample quantiles}
#' \item{Qref}{reference quantile}
#' \item{Qhat}{linear model fit at each quantile}
#' \item{SST}{total sum of squares}
#' \item{SSB}{between sum of squares}
#' \item{SSE}{within sum of squares}
#' \item{roughWeights}{SSE / SST}
#' \item{smoothWeights}{smoothed weights computed using a 
#' running median with a given \code{window} size.}
#' 
#' @aliases qstats
#' 
#' @docType methods
#' @importFrom stats runmed
#' @importFrom stats model.matrix
#'
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' qs <- qstats(object = dat, 
#'              group_factor = rep(c(0,1), each=10), 
#'              window = 0.05)
#' 
#' @rdname qstats
#' @export
qstats <- function(object, group_factor, window = 0.05)
  {
  # Compute sample quantiles
  Q = apply(object, 2, sort) 
  
  # Compute quantile reference
  Qref = rowMeans(Q)  
  
  # Compute SST
  SST = rowSums((Q - Qref)^2)
  
  # Compute SSB
  if(is.factor(group_factor)){
    X = model.matrix(~ 0 + group_factor)
  } else {
    X = model.matrix(~ group_factor)
  }
  
  QBETAS = t(solve(t(X) %*% X) %*% t(X) %*% t(Q))
  Qhat = QBETAS %*% t(X)
  SSB = rowSums((Qhat - Qref)^2)
  
  # Compute weights
  roughWeights = 1 - SSB / SST
  roughWeights[SST < 1e-6] = 1
  
  # Compute smooth weights
  k = floor(window * nrow(Q))
  if (k %% 2 == 0) k = k + 1
  smoothWeights = runmed(roughWeights, k = k, endrule="constant")
  
  list(Q=Q, Qref=Qref, Qhat=Qhat, SST=SST, SSB=SSB, SSE=SST-SSB, 
       roughWeights=roughWeights, smoothWeights=smoothWeights)
}

#' @title Plot weights from \code{qsmooth} function.
#' 
#' @description This function plots a scatterplot
#' showing the \code{qsmoothWeights} along the y-axis
#' and the quantiles on the x-axis.
#' 
#' @param object a qsmooth object from \code{qsmooth} 
#' @param xLab label for x-axis. Default is "quantiles"
#' @param yLab label for y-axis. Default is "weights"
#' @param mainLab title of plot. Default is "qsmooth weights"
#' 
#' @return A scatterplot will be created showing the 
#' \code{qsmoothWeights} along the y-axis and the 
#' quantiles on the x-axis.
#' 
#' @importFrom graphics par
#' @importFrom graphics plot
#' @importFrom graphics abline
#' 
#' @export
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' dat_qs <- qsmooth(object = dat, 
#'                   group_factor = rep(c(0,1), each=10))
#' qsmoothPlotWeights(dat_qs)
#' 
qsmoothPlotWeights <- function(object,
            xLab = "quantiles", yLab = "weights", 
            mainLab = "qsmooth weights"){
    oldpar = par(mfrow=c(1,1), mar=c(4, 4, 1.5, 0.5))
    w = object@qsmoothWeights
    lq = nrow(object@qsmoothData)
    u = (seq_len(lq) - 0.5) / lq
    
    if (length(u) > 1e4) { # do not plot > 10000 points
        sel = sample(seq_len(lq), 1e4)
        plot(u[sel], w[sel], pch=".", main = mainLab,
             xlab = xLab, ylab = yLab, ylim=c(0, 1))
    } else{
        plot(u, w, pch=".", main = mainLab,
             xlab = xLab, ylab = yLab, ylim=c(0, 1))
    }
    abline(h=0.5, v=0.5, col="red", lty=2)
    par(oldpar)
}

#' @title Accessors for the 'qsmoothWeights' slot of a qsmooth object.
#' 
#' @description Accessors for the 'qsmoothWeights' slot of a qsmooth object.
#' 
#' @usage
#' \S4method{qsmoothWeights}{qsmooth}(object)
#'
#' @docType methods
#' @name qsmoothWeights
#' @rdname qsmoothWeights
#' @aliases qsmoothWeights qsmoothWeights,qsmooth-method
#' @param object an object of class \code{qsmooth}.
#' 
#' @return The weights calculated for each feature 
#' after applying smoothed quantile normalization. 
#' 
#' @export
#' 
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' dat_qs <- qsmooth(object = dat, 
#'                   group_factor = rep(c(0,1), each=10))
#' qsmoothWeights(dat_qs)
#' 
setMethod(
  f = "qsmoothWeights", 
  signature = "qsmooth",
  definition = function(object) {
            return(object@qsmoothWeights)
          }
)

#' @title Accessors for the 'qsmoothData' slot of a qsmooth object.
#' 
#' @description Accessors for the 'qsmoothData' slot of a qsmooth object.
#' 
#' @usage
#' \S4method{qsmoothData}{qsmooth}(object)
#'
#' @docType methods
#' @name qsmoothData
#' @rdname qsmoothData
#' @aliases qsmoothData qsmoothData,qsmooth-method
#' @param object an object of class \code{qsmooth}.
#' 
#' @return The normalized data after applying 
#' smoothed quantile normalization. 
#' 
#' @export
#' 
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' dat_qs <- qsmooth(object = dat, 
#'                   group_factor = rep(c(0,1), each=10))
#' qsmoothData(dat_qs)
#' 
setMethod(
  f = "qsmoothData", 
  signature = "qsmooth",
  definition = function(object) {
            return(object@qsmoothData)
          }
)

#' @title the qsmooth class
#'
#' @description  Objects of this class store all 
#' the values needed information to work with a
#' qsmooth object
#' 
#' @slot qsmoothWeights qsmooth weights
#' @slot qsmoothData qsmooth normalized data
#' 
#' @return \code{qsmoothWeights} returns the qsmooth weights and 
#' \code{qsmoothData} returns the qsmooth normalized data 
#' 
#' @name qsmooth-class
#' @import methods
#' @exportClass qsmooth
#' @aliases qsmooth-class
#'
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' dat_qs <- qsmooth(object = dat, 
#'                   group_factor = rep(c(0,1), each=10))
#'
setClass(
    Class = "qsmooth", 
      slot = list(
        qsmoothWeights = "numeric", qsmoothData = "matrix")
)

#' @importFrom utils head
#' @importFrom utils tail
setMethod("show", "qsmooth", 
          function(object){
            cat("qsmooth: Smooth quantile normalization\n")
            cat("   qsmoothWeights (length =", 
                length(object@qsmoothWeights), "):", "\n")
            cat(c(head(round(object@qsmoothWeights,3), n=3), "...", 
                   tail(round(object@qsmoothWeights,3), n=3)), "\n")
            cat("   qsmoothData (nrows =", 
                dim(object@qsmoothData)[1], ", ncols =", 
                dim(object@qsmoothData)[2], "):", "\n")
            print(head(object@qsmoothData))
            if(dim(object@qsmoothData)[1] > 6){
              cat(" ....... \n") }
          }
)

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----fig.cap="Figure 1", out.width = '100%'-----------------------------------
knitr::include_graphics("qsmooth_algo.jpg")

## ----load-lib, message=FALSE--------------------------------------------------
library(qsmooth)

## ----data-1, message=FALSE, warning=FALSE-------------------------------------
library(SummarizedExperiment)
library(bodymapRat)
bm_dat <- bodymapRat()

# select brain and liver samples, stage 21 weeks, and only bio reps
keepColumns = (colData(bm_dat)$organ %in% c("Brain", "Liver")) &
         (colData(bm_dat)$stage == 21) & (colData(bm_dat)$techRep == 1)
keepRows = rowMeans(assay(bm_dat)) > 10 # Filter out low counts
bm_dat_e1 <- bm_dat[keepRows,keepColumns]
bm_dat_e1

## ----calculate-qsmooth1, fig.height=10, fig.width=10--------------------------
library(quantro)

par(mfrow=c(2,2))
pd1 <- colData(bm_dat_e1)
counts1 <- assay(bm_dat_e1)[!grepl("^ERCC", 
                      rownames( assay(bm_dat_e1))), ]
pd1$group <- paste(pd1$organ, pd1$sex, sep="_")

matboxplot(log2(counts1+1), groupFactor = factor(pd1$organ),
           main = "Raw data", xaxt="n", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(as.character(pd1$organ))),
     labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(counts1+1), groupFactor = pd1$organ, 
           main = "Raw data", ylab= "density",
           xlab = "Expression (log2 scale)")
legend('topright', levels(factor(pd1$organ)), 
       col = 1:2, lty = 1)

qs_norm_e1 <- qsmooth(object = counts1, group_factor = pd1$organ)
qs_norm_e1 

matboxplot(log2(qsmoothData(qs_norm_e1)+1), 
           groupFactor = pd1$organ, xaxt="n",
           main = "qsmooth normalized data", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(pd1$organ)), labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(qsmoothData(qs_norm_e1)+1), groupFactor = pd1$organ,
           main = "qsmooth normalized data",
           xlab = "Expression (log2 scale)", ylab = "density")
legend('topright', levels(factor(pd1$organ)), col = 1:2, lty = 1)

## ----plot-qsmooth1-weights----------------------------------------------------
qsmoothPlotWeights(qs_norm_e1)

## -----------------------------------------------------------------------------
par(mfrow=c(2,2))
pd1 <- colData(bm_dat_e1)
counts1 <- assay(bm_dat_e1)[!grepl("^ERCC", 
                      rownames( assay(bm_dat_e1))), ]
pd1$group <- paste(pd1$organ, pd1$sex, sep="_")

matboxplot(log2(counts1+1), groupFactor = factor(pd1$organ),
           main = "Raw data", xaxt="n", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(as.character(pd1$organ))),
     labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(counts1+1), groupFactor = pd1$organ, 
           main = "Raw data", ylab= "density",
           xlab = "Expression (log2 scale)")
legend('topright', levels(factor(pd1$organ)), 
       col = 1:2, lty = 1)

# retrieved GC-content using EDASeq:
# gc <- EDASeq::getGeneLengthAndGCContent(id = rownames(bm_dat_e1), 
#                                   org = "rno")
data(gc, package="qsmooth")

gcContent <- gc[rownames(counts1),2]
keep <- names(gcContent)[!is.na(gcContent)]
qs_norm_gc <- qsmoothGC(object = counts1[keep,], gc=gcContent[keep], 
                        group_factor = pd1$organ)
qs_norm_gc 

matboxplot(log2(qsmoothData(qs_norm_gc)+1), 
           groupFactor = pd1$organ, xaxt="n",
           main = "qsmoothGC normalized data", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(pd1$organ)), labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(qsmoothData(qs_norm_gc)+1), groupFactor = pd1$organ,
           main = "qsmoothGC normalized data",
           xlab = "Expression (log2 scale)", ylab = "density")
legend('topright', levels(factor(pd1$organ)), col = 1:2, lty = 1)

## ----session-info-------------------------------------------------------------
sessionInfo()

#' Generic function that returns the qsmooth weights
#'
#' Given a qsmooth object, this function returns the 
#' qsmooth weights
#' @rdname qsmoothWeights
setGeneric(
  name = "qsmoothWeights", 
  def = function(object) { 
    standardGeneric("qsmoothWeights") 
  }
)

#' Generic function that returns the qsmooth normalized data
#'
#' Given a qsmooth object, this function returns the 
#' qsmooth normalized data
#' @rdname qsmoothData
setGeneric(
  name = "qsmoothData", 
  def = function(object) { 
    standardGeneric("qsmoothData") 
  }
)

.smooth_gcqn <- function(counts, 
                         gc,
                         group_factor, 
                         nGroups=50,
                         round=TRUE,
                         ...){
  gc_groups <- Hmisc::cut2(gc, g = nGroups)
  gcBinNormCounts <- matrix(NA, nrow=nrow(counts), ncol=ncol(counts), 
                            dimnames=list(rownames(counts),colnames(counts)))
  for(ii in seq_len(nlevels(gc_groups))){
    id <- which(gc_groups==levels(gc_groups)[ii])
    countBin <- counts[id,]
    qs <- qsmooth::qsmooth(countBin, group_factor=group_factor, ...)
    normCountBin <- qsmooth::qsmoothData(qs)
    if(round) normCountBin <- base::round(normCountBin)
    normCountBin[normCountBin<0] <- 0
    gcBinNormCounts[id,] <- normCountBin
  }
  return(gcBinNormCounts)
}


#' @title qsmoothGC
#' 
#' @description This function applies smoothed quantile 
#' normalization separately for groups of features that are
#' binned according to their GC-content.
#'
#' @param object an object which is a \code{matrix} or 
#' \code{data.frame} with observations (e.g. probes or genes) on 
#' the rows and samples as the columns. Alternatively, 
#' a user can provide a \code{SummarizedExperiment} object
#' and the \code{assay(object, "counts")} will be used as input 
#' for the qsmooth normalization.
#' @param gc GC-content of the features, ordered according to the features 
#' in \code{object}.
#' @param group_factor a group level continuous or categorial 
#' covariate associated with each sample or column in the 
#' \code{object}. The order of the \code{group_factor} must 
#' match the order of the columns in \code{object}.
#' @param nGroups The number of equally-sized bins used to group the 
#' GC-content values. Groups are created using \code{Hmisc::cut2}.
#' @param round Should normalized values be rounded to integers?
#' @param ... (Optional) Additional arguments passed to \code{\link{qsmooth}}.
#' @export 
#' 
#' @return A matrix of normalized counts.
#' 
#' @aliases qsmoothGC
#' 
#' @docType methods
#' @name qsmoothGC
#' @importFrom sva ComBat
#' @importFrom SummarizedExperiment assays
#' @importFrom SummarizedExperiment assay
#' @importFrom stats ave
#' @importFrom Hmisc cut2
#' 
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' gc <- runif(n=100, min=0.2, max=0.9)
#' dat_qs <- qsmoothGC(object = dat, 
#'                    gc = gc,
#'                    group_factor = rep(c(0,1), each=10))
#' 
#' @references 
#' Van den Berge K., Chou H., Roux de Bézieux H., Street K., Risso D., Ngai J., Dudoit S. Normalization benchmark of ATAC-seq datasets shows the 
#' importance of accounting for GC-content effects. https://www.biorxiv.org/content/10.1101/2021.01.26.428252v2
#' 
#' @rdname qsmoothGC
#' @export
qsmoothGC <- function(object, 
                      group_factor,
                      gc,
                      nGroups = 50,
                      round = TRUE,
                      ...){
  
  if(!any(is(object, "matrix") | is(object, "data.frame") |
          is(object, "SummarizedExperiment"))){
    stop("The class of the object must be a matrix, 
         data.frame or SummarizedExperiment")
  }
  
  if(is.data.frame(object)){ object <- as.matrix(object) }
  
  if(is(object, "SummarizedExperiment")){ 
    if("counts" %in% names(assays(object))){ 
      object <- assay(object, "counts")
    } else {
      stop("There is no assay slot named 'counts' inside 
           the object. Please check the names of the 
           assay slots using names(assays(object)).")
    }
  }
  
  objectNorm <- .smooth_gcqn(counts = object,
                             gc = gc,
                             group_factor = group_factor,
                             nGroups = nGroups, 
                             round = round,
                             ...)
  results <- new("qsmooth")
  results@qsmoothData <- objectNorm
  return(results)
}

library(testthat)
context("checking the qsmooth function")

library(qsmooth)

test_that("checking dimensions of qsmooth object", {
  n_genes <- 1000
  n_samples <- 10
  set.seed(1234)
  x <- matrix(rnorm(n_genes*n_samples), n_genes, n_samples)
  group_factor <- rep(c(1,2),each=5)
  qs_norm <- qsmooth(object = x, 
                    group_factor = group_factor)
  
  expect_equal(length(qsmoothWeights(qs_norm)), n_genes)
  expect_equal(nrow(qsmoothData(qs_norm)), n_genes)
  expect_equal(ncol(qsmoothData(qs_norm)), n_samples)
})

test_that("checking qsmooth normalized data", {
  n_genes <- 1000
  n_samples <- 10
  set.seed(1234)
  x <- matrix(rnorm(n_genes*n_samples), n_genes, n_samples)
  group_factor <- rep(c(1,2),each=5)
  qs_norm <- qsmooth(object = x, group_factor = group_factor)
  expect_equal(qsmoothData(qs_norm)[1,1], -1.25856, tolerance = .0001)
  expect_equal(qsmoothWeights(qs_norm)[1], 0.97495, tolerance = .0001)
})


test_that("checking qsmoothGC normalized data", {
  n_genes <- 1000
  n_samples <- 10
  set.seed(1234)
  x <- matrix(rpois(n_genes*n_samples, lambda=12), n_genes, n_samples)
  group_factor <- rep(c(1,2),each=5)
  gc <- runif(n=n_genes, min=0.2, max=0.9)
  ## one group should be same as qsmooth
  qs_norm <- qsmooth(object = x, group_factor = group_factor)
  qs_normGC <- qsmoothGC(object = x, 
                         gc = gc, 
                         group_factor = group_factor, 
                         nGroups = 1,
                         round = FALSE)
  expect_equal(qsmoothData(qs_norm), qsmoothData(qs_normGC), tolerance = .001)
})

#' @title qsmooth
#' 
#' @description This function applies a generalization of 
#' quantile normalization called smoothed quantile 
#' normalization. This function defines the qsmooth class 
#' and constructor. 
#'
#' @param object an object which is a \code{matrix} or 
#' \code{data.frame} with observations (e.g. probes or genes) on 
#' the rows and samples as the columns. Alternatively, 
#' a user can provide a \code{SummarizedExperiment} object
#' and the \code{assay(object, "counts")} will be used as input 
#' for the qsmooth normalization. 
#' @param group_factor a group level continuous or categorial 
#' covariate associated with each sample or column in the 
#' \code{object}. The order of the \code{group_factor} must 
#' match the order of the columns in \code{object}.
#' @param batch (Optional) batch covariate (multiple 
#' batches are not allowed). 
#' If batch covariate is provided, \code{Combat()} from 
#' \code{sva} is used prior to qsmooth normalization to 
#' remove batch effects. See \code{Combat()} for more details. 
#' @param norm_factors optional normalization scaling factors.
#' @param window window size for running median which is 
#' a fraction of the number of rows in \code{object}. 
#' Default is 0.05. 
#' @export 
#' 
#' @return A object of the class \code{qsmooth} that 
#' contains a numeric vector of the qsmooth weights in 
#' the \code{qsmoothWeights} slot and a matrix of normalized  
#' values after applying smoothed quantile normalization in 
#' the \code{qsmoothData} slot. 
#' 
#' @details 
#' Quantile normalization is one of the most widely used 
#' normalization tools for data analysis in genomics. Although it 
#' was originally developed for gene expression microarrays it is
#' now used across many different high-throughput applications 
#' including RNAseq and ChIPseq. The methodology 
#' relies on the assumption that observed changes in the empirical 
#' distribution of samples are due to unwanted variability. 
#' Because the data is transformed to remove these differences 
#' it has the potential to remove interesting biologically driven
#' global variation. Therefore, applying quantile normalization, 
#' or other global normalization methods that rely on similar 
#' assumptions, may not be an appropriate depending on the type 
#' and source of variation. 
#' 
#' This function computes a weight at every 
#' quantile that compares the variability between groups 
#' relative to within groups. In one extreme quantile 
#' normalization is applied and in the other extreme quantile 
#' normalization within each biological condition is applied. 
#' The weight shrinks the group-level quantile normalized data 
#' towards the overall reference quantiles if variability 
#' between groups is sufficiently smaller than the variability 
#' within groups. 
# 
#' See the vignette for more details. 
#' 
#' @aliases qsmooth
#' 
#' @docType methods
#' @name qsmooth
#' @importFrom sva ComBat
#' @importFrom SummarizedExperiment assays
#' @importFrom SummarizedExperiment assay
#' @importFrom stats ave
#' 
#' @examples
#' dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
#'              matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
#' dat_qs <- qsmooth(object = dat, 
#'                   group_factor = rep(c(0,1), each=10))
#' 
#' @rdname qsmooth
#' @export
qsmooth <- function(object, group_factor,
                    batch = NULL, norm_factors = NULL, 
                    window = 0.05)
{

  if(!any(is(object, "matrix") | is(object, "data.frame") |
     is(object, "SummarizedExperiment"))){
    stop("The class of the object must be a matrix, 
         data.frame or SummarizedExperiment")
  }
  
  if(is.data.frame(object)){ object <- as.matrix(object) }
  
  if(is(object, "SummarizedExperiment")){ 
    if("counts" %in% names(assays(object))){ 
      object <- assay(object, "counts")
    } else {
      stop("There is no assay slot named 'counts' inside 
           the object. Please check the names of the 
           assay slots using names(assays(object)).")
    }
  }
  
  if(is.null(group_factor)){  
    stop("Must provide group_factor to specify the group 
        level information associated with each sample or 
             or column in object.")
  }
  
  if(ncol(object) != length(group_factor)){
    stop("Number of columns in object does not match 
             length of group_factor.")
  }
  
  if(is.factor(group_factor) & length(levels(group_factor)) < 2){
    stop(paste0("group_factor is a factor and number of levels in 
                  group_factor is less than 2 (levels(group_factor): ",
                levels(group_factor), "). Must provide a factor with 
                  2 or more levels to use qsmooth."))
  }
  
  if(any(is.na(object))){ stop("Object must not contains NAs.") }    
  
  # Scale normalization step
  if(!is.null(norm_factors)) { 
    object <- sweep(object, 2, norm_factors, FUN = "/") 
  }
  
  # If batch is provided, run sva::ComBat to remove batch effects
  if(!is.null(batch)){
    object <- ComBat(object, factor(batch))
  }
  
  # Compute quantile statistics
  qs <- qstats(object=object, group_factor=group_factor,
               window=window)
  Qref <- qs$Qref 
  Qhat <- qs$Qhat  
  w <- qs$smoothWeights
  
  # Weighted quantiles
  objectNorm = w * Qref + (1 - w) * Qhat
  
  # Re-order objectNorm by rank of object (columnwise)
  for (i in seq_len(ncol(objectNorm))) {
    # Grab ref. i
    ref = objectNorm[,i]
    
    # Grab object column i
    x = object[,i]
    
    # Grab ranks of x (using min rank for ties)
    rmin = rank(x, ties.method="min")
    
    # If x has rank ties then average the values of ref at 
    # those ranks
    dups = duplicated(rmin)
    
    if (any(dups)) {
      # Grab ranks of x (using random ranks for ties) 
      # (needed to uniquely identify the indices of tied ranks)
      rrand = rank(x, ties.method="random")
      
      # Grab tied ranks
      tied.ranks = unique(rmin[dups])
      
      for (k in tied.ranks) {
        # Select the indices of tied ranks 
        sel = rrand[rmin == k] 
        ref[sel] = ave(ref[sel])
      }
    }
    
    # Re-order ref and replace in objectNorm
    objectNorm[,i] = ref[rmin]
  }
  
  results <- new("qsmooth")
  results@qsmoothWeights <- w
  
  rownames(objectNorm) = rownames(object)
  colnames(objectNorm) = colnames(object)
  results@qsmoothData <- objectNorm
  
  return(results)
}

---
title: "The qsmooth user's guide"
author:
- name: Stephanie C. Hicks
  affiliation: Johns Hopkins Bloomberg School of Public Health
- name: Kwame Okrah
  affiliation: Genentech
- name: Joseph Paulson 
  affiliation: Genentech
- name: John Quackenbush 
  affiliation: Harvard T.H. Chan School of Public Health
- name: Rafael A. Irizarry 
  affiliation: Dana-Farber Cancer Institute
- name: Hector Corrada-Bravo
  affiliation: University of Maryland
output:
  BiocStyle::html_document:
    toc_float: true
package: qsmooth
abstract: |
 Smooth quantile normalization
   is a generalization of quantile normalization, 
   which is average of the two types of assumptions 
   about the data generation process: quantile normalization
   and quantile normalization between groups.
vignette: |
  %\VignetteIndexEntry{The qsmooth user's guide}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, results="hide", message=FALSE}
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```


## Introduction

Global normalization methods such as quantile normalization
[@Bolstad2003], [@Irizarry2003] have become a standard part of the
analysis pipeline for high-throughput data to remove unwanted
technical variation. These methods and others that rely solely
on observed data without external information (e.g. spike-ins)
are based on the assumption that only a minority of genes are
expected to be differentially expressed (or that an equivalent
number of genes increase and decrease across biological conditions
[@aanes2014normalization]). This assumption can be interpreted
in different ways leading to different global normalization
procedures. For example, in one normalization procedure, the method assumes
the mean expression level across genes should be the same across samples
[@Robinson2010]. In contrast, quantile normalization assumes the
only difference between the statistical distribution of each sample
is technical variation. Normalization is achieved by forcing the
observed distributions to be the same and the average distribution,
obtained by taking the average of each quantile across samples,
is used as the reference [@Bolstad2003].

While these assumptions may be reasonable in certain experiments,
they may not always be appropriate [@Loven2012], [@Hicks2015].
For example, mRNA content has been shown to fluctuate significantly
during zebrafish early developmental stages [@aanes2014normalization].
Similarly, cells expressing high levels of c-Myc undergo
transcriptional amplification causing a 2 to 3 fold change in global
gene expression compared to cells expressing low c-Myc
[@Loven2012].

Recently, an R/Bioconductor package (`quantro`) [@Hicks2015]
has been developed to test for global differences between groups of
distributions to evaluate whether global normalization methods such
as quantile normalization should be applied. If global differences
are found between groups of distributions, these changes may be of technical
or biological of interest. If these changes are of technical interest
(e.g. batch effects), then global normalization methods should be applied.
If these changes are related to a biological covariate (e.g. normal/tumor or
two tissues), then global normalization methods should not be applied
because the methods will remove the interesting biological variation
(i.e. differentially expressed genes) and artifically induce differences
between genes that were not differentially expressed. In the cases
with global differences between groups of distributions
between biological conditions, quantile normalization is
not an appropriate normalization method. In
these cases, we can consider a more relaxed assumption about the data,
namely that the statistical distribution of each sample should be the
same within biological conditions or groups (compared to the more
stringent assumption of quantile normalization, which states the
statistical distribution is the same across all samples).

In this vignette we introduce a generalization of quantile
normalization, referred to as **smooth quantile normalization**
(**qsmooth**), which is a weighted average of the two
types of assumptions about the data. The `qsmooth` R-package
contains the `qsmooth()` function, which computes a weight at
every quantile that compares the variability between groups relative
to within groups. In one extreme, quantile normalization is applied
and in the other extreme quantile normalization within each
biological condition is applied. The weight shrinks the group-level
quantile normalized data towards the overall reference quantiles
if variability between groups is sufficiently smaller than the
variability within groups. The algorithm is described in
Figure 1 below.

Let `gene(g)` denote the ${g}^{th}$ row after sorting
each column in the data. For each row, `gene(g)`, we
compute the weight $w_{(g)} \in [0, 1]$, where a weight of 0 implies
quantile normalization within groups is applied and
a weight of 1 indicates quantile normalization is applied.
The weight at each row depends on the between group sum of squares
$\hbox{SSB}_{(g)}$ and total sum of squares $\hbox{SST}_{(g)}$,
as follows:

$$
w_{(g)} = \hbox{median} \{1 - \hbox{SSB}_{(i)} / \hbox{SST}_{(i)} 
            | ~i = g-k, \dots, g, \dots, g+k \} 
$$
where $k=$ floor(Total number of genes * 0.05). The number
0.05 is a flexible parameter that can be altered to change the
window of the number of genes considered. In this way, we
can use a rolling median to borrow information from
neighboring genes in the weight.

```{r, fig.cap="Figure 1", out.width = '100%'}
knitr::include_graphics("qsmooth_algo.jpg")
```


# Getting Started

Load the package in R

```{r load-lib, message=FALSE}
library(qsmooth)
```

# Data

The **bodymapRat** package contains an
`SummarizedExperiment` of 652 RNA-Seq samples 
from a comprehensive rat transcriptomic BodyMap study. 
This data was derived from the raw FASTQ files obtained
from Yu et al. (2014) \cite{Yu2014}. It contains expression 
levels from 11 organs in male and female rats at 4
developmental stages. We will use a subset of this data in
this vignette.

## bodymapRat example - Comparing two tissues

This example is based a dataset which contains
brain and liver tissue samples from 21 week old male 
and female rats. eight samples are from males and 
eight samples are from females.

```{r data-1, message=FALSE, warning=FALSE}
library(SummarizedExperiment)
library(bodymapRat)
bm_dat <- bodymapRat()

# select brain and liver samples, stage 21 weeks, and only bio reps
keepColumns = (colData(bm_dat)$organ %in% c("Brain", "Liver")) &
         (colData(bm_dat)$stage == 21) & (colData(bm_dat)$techRep == 1)
keepRows = rowMeans(assay(bm_dat)) > 10 # Filter out low counts
bm_dat_e1 <- bm_dat[keepRows,keepColumns]
bm_dat_e1
```

# Using the `qsmooth()` function

## Input for `qsmooth()`

The `qsmooth()` function must have two objects as input:

1. `object`: a data frame or matrix with observations
(e.g. probes or genes) on the rows and samples as the 
columns. `qsmooth()` accepts objects which are a data 
frame or matrix with observations (e.g. probes or genes) 
on the rows and samples as the columns.
2. `group_factor`: a continuous or categorial covariate
that represents the group level biological variation 
about each sample. For example if the samples 
represent two different tissues, provide `qsmooth()` 
with a covariate representing which columns in the 
`object` are different tissue samples.
2. `batch`: **optional** batch covariate (multiple
batches are not allowed). If batch covariate is provided,
`ComBat()` from `sva` is used prior to
qsmooth normalization to remove batch effects.
See `ComBat()` for more details.
3. `norm_factors`: **optional** scaling normalization factors.
Default is `NULL`. If `norm_factors` is not equal to
`NULL`, the user can provide a vector of scaling factors
that will be used to modify the expression data set prior to
applying the `qsmooth` algorithm.
4. `window`: window size for running median (defined as a
fraction of the number of rows of the data object.
Default is 0.05. 



## Running `qsmooth()`

### bodymapRat example - Comparing two tissues

Here, the groups we are interested in comparing 
are the two types of tissues in the 21 week old
male and female rats. The groups we are interested 
in comparing is contained in the `organ` 
column in the `colData(bm_dat_e1)` dataset. To run the 
`qsmooth()` function, input the data object and the
object containing the phenotypic data. 

The first row shows the boxplots and density plot
of the raw data that has been transformed on the 
`log2()` scale and added a pseudo-count of 1 (i.e. 
`log2(counts+1)`). 

The second row shows the boxplots and density plot 
of the qsmooth normalized data.

```{r calculate-qsmooth1, fig.height=10, fig.width=10}
library(quantro)

par(mfrow=c(2,2))
pd1 <- colData(bm_dat_e1)
counts1 <- assay(bm_dat_e1)[!grepl("^ERCC", 
                      rownames( assay(bm_dat_e1))), ]
pd1$group <- paste(pd1$organ, pd1$sex, sep="_")

matboxplot(log2(counts1+1), groupFactor = factor(pd1$organ),
           main = "Raw data", xaxt="n", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(as.character(pd1$organ))),
     labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(counts1+1), groupFactor = pd1$organ, 
           main = "Raw data", ylab= "density",
           xlab = "Expression (log2 scale)")
legend('topright', levels(factor(pd1$organ)), 
       col = 1:2, lty = 1)

qs_norm_e1 <- qsmooth(object = counts1, group_factor = pd1$organ)
qs_norm_e1 

matboxplot(log2(qsmoothData(qs_norm_e1)+1), 
           groupFactor = pd1$organ, xaxt="n",
           main = "qsmooth normalized data", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(pd1$organ)), labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(qsmoothData(qs_norm_e1)+1), groupFactor = pd1$organ,
           main = "qsmooth normalized data",
           xlab = "Expression (log2 scale)", ylab = "density")
legend('topright', levels(factor(pd1$organ)), col = 1:2, lty = 1)
```

The smoothed quantile normalized data can be 
extracted using the` qsmoothData()` function
(see above) and the smoothed quantile weights
can plotted using the `qsmoothPlotWeights()` 
function (see below).

```{r plot-qsmooth1-weights}
qsmoothPlotWeights(qs_norm_e1)
```

The weights are calculated for each quantile 
in the data set. A weight of 1 indicates 
quantile normalization is applied, where as a 
weight of 0 indicates quantile normalization
within the groups is applied. See the Figure 1 
for more details on the weights.


In this example, the weights range from 0.2 
to 0.8 across the quantiles, where the weights 
are close to 0.2 for the quantiles close to 0 
and the weights are close to 0.8 for the quantiles 
close to 1. This plot suggests the distributions
contain more variablity between the groups compared 
to within groups for the small quantiles (and the 
conventional quantile normalization is not 
necessarily appropriate). As the quantiles get 
bigger, there is less variability between groups 
which increases the weight closer to 0.8 as the 
quantiles get bigger.

However, if the weights are close to 1 
across all the quantiles, this indicates that there 
is no major difference between the group-level
quantiles in the two groups 

## Running `qsmoothGC`

In many high-throughput experiments, a sample-specific GC-content effect
can be observed. Due to this technical variability that differs between samples,
it can obscure comparisons across (sets of) samples.
[@VandenBerge2021] propose a method based on smooth quantile normalization,
where the `qsmooth` procedure is adopted separately across bins of features that
have been grouped according to their GC-content values.
This procedure has been implemented in `qsmoothGC`, which can be used in
similar vein as the `qsmooth` function. It has three additional arguments:
 - `gc` to define the GC-content values of each feature.
 - `nGroups` to define the number of groups within which `qsmooth` normalization
 is adopted.
 - `round` to define whether the normalized data should be rounded.
 
```{r}
par(mfrow=c(2,2))
pd1 <- colData(bm_dat_e1)
counts1 <- assay(bm_dat_e1)[!grepl("^ERCC", 
                      rownames( assay(bm_dat_e1))), ]
pd1$group <- paste(pd1$organ, pd1$sex, sep="_")

matboxplot(log2(counts1+1), groupFactor = factor(pd1$organ),
           main = "Raw data", xaxt="n", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(as.character(pd1$organ))),
     labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(counts1+1), groupFactor = pd1$organ, 
           main = "Raw data", ylab= "density",
           xlab = "Expression (log2 scale)")
legend('topright', levels(factor(pd1$organ)), 
       col = 1:2, lty = 1)

# retrieved GC-content using EDASeq:
# gc <- EDASeq::getGeneLengthAndGCContent(id = rownames(bm_dat_e1), 
#                                   org = "rno")
data(gc, package="qsmooth")

gcContent <- gc[rownames(counts1),2]
keep <- names(gcContent)[!is.na(gcContent)]
qs_norm_gc <- qsmoothGC(object = counts1[keep,], gc=gcContent[keep], 
                        group_factor = pd1$organ)
qs_norm_gc 

matboxplot(log2(qsmoothData(qs_norm_gc)+1), 
           groupFactor = pd1$organ, xaxt="n",
           main = "qsmoothGC normalized data", 
           ylab = "Expression (log2 scale)")
axis(1, at=seq_len(length(pd1$organ)), labels=FALSE)
text(seq_len(length(pd1$organ)), par("usr")[3] -2, 
     labels = pd1$organ, srt = 90, pos = 1, xpd = TRUE)

matdensity(log2(qsmoothData(qs_norm_gc)+1), groupFactor = pd1$organ,
           main = "qsmoothGC normalized data",
           xlab = "Expression (log2 scale)", ylab = "density")
legend('topright', levels(factor(pd1$organ)), col = 1:2, lty = 1)
```


# References

# Session Info

```{r session-info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{qsmoothWeights}
\alias{qsmoothWeights}
\alias{qsmoothWeights,qsmooth-method}
\title{Generic function that returns the qsmooth weights}
\usage{
qsmoothWeights(object)

\S4method{qsmoothWeights}{qsmooth}(object)
}
\arguments{
\item{object}{an object of class \code{qsmooth}.}
}
\value{
The weights calculated for each feature 
after applying smoothed quantile normalization.
}
\description{
Given a qsmooth object, this function returns the 
qsmooth weights

Accessors for the 'qsmoothWeights' slot of a qsmooth object.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
dat_qs <- qsmooth(object = dat, 
                  group_factor = rep(c(0,1), each=10))
qsmoothWeights(dat_qs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{qsmoothPlotWeights}
\alias{qsmoothPlotWeights}
\title{Plot weights from \code{qsmooth} function.}
\usage{
qsmoothPlotWeights(
  object,
  xLab = "quantiles",
  yLab = "weights",
  mainLab = "qsmooth weights"
)
}
\arguments{
\item{object}{a qsmooth object from \code{qsmooth}}

\item{xLab}{label for x-axis. Default is "quantiles"}

\item{yLab}{label for y-axis. Default is "weights"}

\item{mainLab}{title of plot. Default is "qsmooth weights"}
}
\value{
A scatterplot will be created showing the 
\code{qsmoothWeights} along the y-axis and the 
quantiles on the x-axis.
}
\description{
This function plots a scatterplot
showing the \code{qsmoothWeights} along the y-axis
and the quantiles on the x-axis.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
dat_qs <- qsmooth(object = dat, 
                  group_factor = rep(c(0,1), each=10))
qsmoothPlotWeights(dat_qs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsmoothGC.R
\docType{methods}
\name{qsmoothGC}
\alias{qsmoothGC}
\title{qsmoothGC}
\usage{
qsmoothGC(object, group_factor, gc, nGroups = 50, round = TRUE, ...)
}
\arguments{
\item{object}{an object which is a \code{matrix} or 
\code{data.frame} with observations (e.g. probes or genes) on 
the rows and samples as the columns. Alternatively, 
a user can provide a \code{SummarizedExperiment} object
and the \code{assay(object, "counts")} will be used as input 
for the qsmooth normalization.}

\item{group_factor}{a group level continuous or categorial 
covariate associated with each sample or column in the 
\code{object}. The order of the \code{group_factor} must 
match the order of the columns in \code{object}.}

\item{gc}{GC-content of the features, ordered according to the features 
in \code{object}.}

\item{nGroups}{The number of equally-sized bins used to group the 
GC-content values. Groups are created using \code{Hmisc::cut2}.}

\item{round}{Should normalized values be rounded to integers?}

\item{...}{(Optional) Additional arguments passed to \code{\link{qsmooth}}.}
}
\value{
A matrix of normalized counts.
}
\description{
This function applies smoothed quantile 
normalization separately for groups of features that are
binned according to their GC-content.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
gc <- runif(n=100, min=0.2, max=0.9)
dat_qs <- qsmoothGC(object = dat, 
                   gc = gc,
                   group_factor = rep(c(0,1), each=10))

}
\references{
Van den Berge K., Chou H., Roux de Bézieux H., Street K., Risso D., Ngai J., Dudoit S. Normalization benchmark of ATAC-seq datasets shows the 
importance of accounting for GC-content effects. https://www.biorxiv.org/content/10.1101/2021.01.26.428252v2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsmooth.R
\docType{methods}
\name{qsmooth}
\alias{qsmooth}
\title{qsmooth}
\usage{
qsmooth(object, group_factor, batch = NULL, norm_factors = NULL, window = 0.05)
}
\arguments{
\item{object}{an object which is a \code{matrix} or 
\code{data.frame} with observations (e.g. probes or genes) on 
the rows and samples as the columns. Alternatively, 
a user can provide a \code{SummarizedExperiment} object
and the \code{assay(object, "counts")} will be used as input 
for the qsmooth normalization.}

\item{group_factor}{a group level continuous or categorial 
covariate associated with each sample or column in the 
\code{object}. The order of the \code{group_factor} must 
match the order of the columns in \code{object}.}

\item{batch}{(Optional) batch covariate (multiple 
batches are not allowed). 
If batch covariate is provided, \code{Combat()} from 
\code{sva} is used prior to qsmooth normalization to 
remove batch effects. See \code{Combat()} for more details.}

\item{norm_factors}{optional normalization scaling factors.}

\item{window}{window size for running median which is 
a fraction of the number of rows in \code{object}. 
Default is 0.05.}
}
\value{
A object of the class \code{qsmooth} that 
contains a numeric vector of the qsmooth weights in 
the \code{qsmoothWeights} slot and a matrix of normalized  
values after applying smoothed quantile normalization in 
the \code{qsmoothData} slot.
}
\description{
This function applies a generalization of 
quantile normalization called smoothed quantile 
normalization. This function defines the qsmooth class 
and constructor.
}
\details{
Quantile normalization is one of the most widely used 
normalization tools for data analysis in genomics. Although it 
was originally developed for gene expression microarrays it is
now used across many different high-throughput applications 
including RNAseq and ChIPseq. The methodology 
relies on the assumption that observed changes in the empirical 
distribution of samples are due to unwanted variability. 
Because the data is transformed to remove these differences 
it has the potential to remove interesting biologically driven
global variation. Therefore, applying quantile normalization, 
or other global normalization methods that rely on similar 
assumptions, may not be an appropriate depending on the type 
and source of variation. 

This function computes a weight at every 
quantile that compares the variability between groups 
relative to within groups. In one extreme quantile 
normalization is applied and in the other extreme quantile 
normalization within each biological condition is applied. 
The weight shrinks the group-level quantile normalized data 
towards the overall reference quantiles if variability 
between groups is sufficiently smaller than the variability 
within groups. 
See the vignette for more details.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
dat_qs <- qsmooth(object = dat, 
                  group_factor = rep(c(0,1), each=10))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qstats.R
\docType{methods}
\name{qstats}
\alias{qstats}
\title{qstats}
\usage{
qstats(object, group_factor, window = 0.05)
}
\arguments{
\item{object}{an object which is a data frame or 
matrix with observations (e.g. probes or genes) on 
the rows and samples as the columns.}

\item{group_factor}{a group level continuous or categorial 
covariate associated with each sample or column in the 
\code{object}. The order of the \code{group_factor} must 
match the order of the columns in \code{object}.}

\item{window}{window size for running median which is 
a fraction of the number of rows in \code{object}. 
Default is 0.05.}
}
\value{
A list of quantile statistics including 
\item{Q}{sample quantiles}
\item{Qref}{reference quantile}
\item{Qhat}{linear model fit at each quantile}
\item{SST}{total sum of squares}
\item{SSB}{between sum of squares}
\item{SSE}{within sum of squares}
\item{roughWeights}{SSE / SST}
\item{smoothWeights}{smoothed weights computed using a 
running median with a given \code{window} size.}
}
\description{
This function is a helper function that 
computes quantile statistics for the function 
\code{qsmooth}.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
qs <- qstats(object = dat, 
             group_factor = rep(c(0,1), each=10), 
             window = 0.05)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{qsmoothData}
\alias{qsmoothData}
\alias{qsmoothData,qsmooth-method}
\title{Generic function that returns the qsmooth normalized data}
\usage{
qsmoothData(object)

\S4method{qsmoothData}{qsmooth}(object)
}
\arguments{
\item{object}{an object of class \code{qsmooth}.}
}
\value{
The normalized data after applying 
smoothed quantile normalization.
}
\description{
Given a qsmooth object, this function returns the 
qsmooth normalized data

Accessors for the 'qsmoothData' slot of a qsmooth object.
}
\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
dat_qs <- qsmooth(object = dat, 
                  group_factor = rep(c(0,1), each=10))
qsmoothData(dat_qs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{qsmooth-class}
\alias{qsmooth-class}
\title{the qsmooth class}
\value{
\code{qsmoothWeights} returns the qsmooth weights and 
\code{qsmoothData} returns the qsmooth normalized data
}
\description{
Objects of this class store all 
the values needed information to work with a
qsmooth object
}
\section{Slots}{

\describe{
\item{\code{qsmoothWeights}}{qsmooth weights}

\item{\code{qsmoothData}}{qsmooth normalized data}
}}

\examples{
dat <- cbind(matrix(rnorm(1000), nrow=100, ncol=10), 
             matrix(rnorm(1000, .1, .7), nrow=100, ncol=10))
dat_qs <- qsmooth(object = dat, 
                  group_factor = rep(c(0,1), each=10))

}
